'use strict';

exports.__esModule = true;

var _nodeBuilder = require('../node-builder');

var _esotope = require('../tools/esotope');

var _replaceNode = require('./replace-node');

var _replaceNode2 = _interopRequireDefault(_replaceNode);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const INVOCATION_FUNC_NAME_RE = /^(call|apply)$/;

// Transform:
// eval.call(ctx, script);
// eval.apply(ctx, script); -->
// eval.call(ctx, __proc$Script(script));
// eval.apply(ctx, __proc$Script(script, true));

// -------------------------------------------------------------
// WARNING: this file is used by both the client and the server.
// Do not use any browser or node-specific API!
// -------------------------------------------------------------

exports.default = {
    nodeReplacementRequireTransform: false,

    nodeTypes: [_esotope.Syntax.CallExpression],

    condition: node => {
        // eval.<meth>(ctx, script, ...)
        if (node.arguments.length < 2) return false;

        if (node.callee.type === _esotope.Syntax.MemberExpression && INVOCATION_FUNC_NAME_RE.test(node.callee.property.name)) {
            const obj = node.callee.object;

            // obj.eval.<meth>(), obj[eval].<meth>(),
            if (obj.type === _esotope.Syntax.MemberExpression && (obj.property.value || obj.property.name) === 'eval') return true;

            // eval.<meth>()
            if (obj.name === 'eval') return true;
        }

        return false;
    },

    run: node => {
        const isApply = node.callee.property.name === 'apply';
        const newArg = (0, _nodeBuilder.createProcessScriptMethCall)(node.arguments[1], isApply);

        (0, _replaceNode2.default)(node.arguments[1], newArg, node, 'arguments');

        return null;
    }
};
module.exports = exports['default'];